// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KI18N_VERSION_H
#define KI18N_VERSION_H

#define KI18N_VERSION_STRING "6.29.0"
#define KI18N_VERSION_MAJOR 6
#define KI18N_VERSION_MINOR 29
#define KI18N_VERSION_PATCH 0
#define KI18N_VERSION ((6<<16)|(29<<8)|(0))

#endif
